"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createAsyncComponent = createAsyncComponent;

var _react = _interopRequireWildcard(require("react"));

var _async = require("@shopify/async");

var _reactIntersectionObserver = require("@shopify/react-intersection-observer");

var _reactIdle = require("@shopify/react-idle");

var _reactHydrate = require("@shopify/react-hydrate");

var _hooks = require("./hooks");

var _types = require("./types");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function createAsyncComponent(_ref) {
  var id = _ref.id,
      load = _ref.load,
      defer = _ref.defer,
      deferHydration = _ref.deferHydration,
      displayName = _ref.displayName,
      _ref$renderLoading = _ref.renderLoading,
      renderLoading = _ref$renderLoading === void 0 ? noopRender : _ref$renderLoading,
      _ref$renderError = _ref.renderError,
      renderError = _ref$renderError === void 0 ? defaultRenderError : _ref$renderError,
      _ref$usePreload = _ref.usePreload,
      useCustomPreload = _ref$usePreload === void 0 ? noopUse : _ref$usePreload,
      _ref$usePrefetch = _ref.usePrefetch,
      useCustomPrefetch = _ref$usePrefetch === void 0 ? noopUse : _ref$usePrefetch,
      _ref$useKeepFresh = _ref.useKeepFresh,
      useCustomKeepFresh = _ref$useKeepFresh === void 0 ? noopUse : _ref$useKeepFresh;
  var resolver = (0, _async.createResolver)({
    id: id,
    load: load
  });
  var componentName = displayName || displayNameFromId(resolver.id);
  var deferred = defer != null;
  var progressivelyHydrated = deferHydration != null;
  var scriptTiming = deferred || progressivelyHydrated ? _types.AssetTiming.CurrentPage : _types.AssetTiming.Immediate;
  var stylesTiming = deferred ? _types.AssetTiming.CurrentPage : _types.AssetTiming.Immediate;

  function Async(props) {
    var _useAsync = (0, _hooks.useAsync)(resolver, {
      scripts: scriptTiming,
      styles: stylesTiming,
      immediate: !deferred
    }),
        Component = _useAsync.resolved,
        load = _useAsync.load,
        loading = _useAsync.loading,
        error = _useAsync.error;

    var _useRef = (0, _react.useRef)((0, _reactHydrate.useHydrationManager)().hydrated),
        startedHydrated = _useRef.current;

    if (error) {
      return renderError(error);
    }

    var loadingMarkup = null;

    if (progressivelyHydrated && !startedHydrated) {
      loadingMarkup = /*#__PURE__*/_react["default"].createElement(Loader, {
        defer: deferHydration,
        load: load,
        props: props
      });
    } else if (loading) {
      loadingMarkup = /*#__PURE__*/_react["default"].createElement(Loader, {
        defer: defer,
        load: load,
        props: props
      });
    }

    var contentMarkup = null;
    var rendered = Component ? /*#__PURE__*/_react["default"].createElement(Component, props) : null;

    if (progressivelyHydrated && !startedHydrated) {
      contentMarkup = rendered ? /*#__PURE__*/_react["default"].createElement(_reactHydrate.Hydrator, {
        id: resolver.id
      }, rendered) : /*#__PURE__*/_react["default"].createElement(_reactHydrate.Hydrator, {
        id: resolver.id
      });
    } else if (loading) {
      contentMarkup = renderLoading(props);
    } else {
      contentMarkup = rendered;
    }

    return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, contentMarkup, loadingMarkup);
  }

  Async.displayName = "Async(".concat(componentName, ")");

  function usePreload(props) {
    var _useAsync2 = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    }),
        load = _useAsync2.load;

    var customPreload = useCustomPreload(props);
    return (0, _react.useCallback)(function () {
      load();

      if (customPreload) {
        customPreload();
      }
    }, [load, customPreload]);
  }

  function usePrefetch(props) {
    var _useAsync3 = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    }),
        load = _useAsync3.load;

    var customPrefetch = useCustomPrefetch(props);
    return (0, _react.useCallback)(function () {
      load();

      if (customPrefetch) {
        customPrefetch();
      }
    }, [load, customPrefetch]);
  }

  function useKeepFresh(props) {
    var _useAsync4 = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    }),
        load = _useAsync4.load;

    var customKeepFresh = useCustomKeepFresh(props);
    return (0, _react.useCallback)(function () {
      load();

      if (customKeepFresh) {
        customKeepFresh();
      }
    }, [load, customKeepFresh]);
  }

  function Preload(options) {
    (0, _reactIdle.useIdleCallback)(usePreload(options));
    return null;
  }

  Preload.displayName = "Async.Preload(".concat(displayName, ")");

  function Prefetch(options) {
    var prefetch = usePrefetch(options);
    (0, _react.useEffect)(function () {
      prefetch();
    }, [prefetch]);
    return null;
  }

  Prefetch.displayName = "Async.Prefetch(".concat(displayName, ")");

  function KeepFresh(options) {
    (0, _reactIdle.useIdleCallback)(useKeepFresh(options));
    return null;
  }

  KeepFresh.displayName = "Async.KeepFresh(".concat(displayName, ")");
  var FinalComponent = Async;
  Reflect.defineProperty(FinalComponent, 'resolver', {
    value: resolver,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Preload', {
    value: Preload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Prefetch', {
    value: Prefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'KeepFresh', {
    value: KeepFresh,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePreload', {
    value: usePreload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePrefetch', {
    value: usePrefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'useKeepFresh', {
    value: useKeepFresh,
    writable: false
  });
  return FinalComponent;
}

function noopUse() {
  return function () {};
}

function noopRender() {
  return null;
}

var DEFAULT_DISPLAY_NAME = 'Component';
var FILENAME_REGEX = /([^/]*)\.\w+$/;

function displayNameFromId(id) {
  if (!id) {
    return DEFAULT_DISPLAY_NAME;
  }

  var match = FILENAME_REGEX.exec(id);
  return match ? match[1] : DEFAULT_DISPLAY_NAME;
}

function defaultRenderError(error) {
  if (error) {
    throw error;
  }

  return null;
}

function Loader(_ref2) {
  var defer = _ref2.defer,
      load = _ref2.load,
      props = _ref2.props;
  var handleIntersection = (0, _react.useCallback)(function (_ref3) {
    var _ref3$isIntersecting = _ref3.isIntersecting,
        isIntersecting = _ref3$isIntersecting === void 0 ? true : _ref3$isIntersecting;

    if (isIntersecting) {
      load();
    }
  }, [load]);
  (0, _react.useEffect)(function () {
    if (defer == null || defer === _async.DeferTiming.Mount) {
      load();
    } else if (typeof defer === 'function' && defer(props)) {
      load();
    }
  }, [defer, load, props]);

  if (typeof defer === 'function') {
    return null;
  }

  switch (defer) {
    case _async.DeferTiming.Idle:
      return /*#__PURE__*/_react["default"].createElement(_reactIdle.OnIdle, {
        perform: load
      });

    case _async.DeferTiming.InViewport:
      return /*#__PURE__*/_react["default"].createElement(_reactIntersectionObserver.IntersectionObserver, {
        threshold: 0,
        onIntersectionChange: handleIntersection
      });

    default:
      return null;
  }
}