import React, { useEffect } from 'react';
import { createResolver } from '@shopify/async';
import { useIdleCallback } from '@shopify/react-idle';
import { useAsync } from './hooks';
import { AssetTiming } from './types';
export function createAsyncContext(_ref) {
  var id = _ref.id,
      load = _ref.load;
  var resolver = createResolver({
    id: id,
    load: load
  });
  var Context = /*#__PURE__*/React.createContext(null); // Just like a "normal" value returned from `createContext`, rendering
  // the value itself is not supported. This component is just a placeholder
  // to provide a more useful error.

  function Root() {
    throw new Error('Do not attempt to render the result of calling `createAsyncContext()` directly. Render its `.Provider` component instead.');
  }

  function Provider(props) {
    var _useAsync = useAsync(resolver, {
      assets: AssetTiming.Immediate
    }),
        load = _useAsync.load,
        resolved = _useAsync.resolved;

    useEffect(function () {
      load();
    }, [load]);
    return /*#__PURE__*/React.createElement(Context.Provider, Object.assign({
      value: resolved
    }, props));
  }

  function Consumer(props) {
    return /*#__PURE__*/React.createElement(Context.Consumer, props);
  }

  function usePreload() {
    return useAsync(resolver, {
      assets: AssetTiming.NextPage
    }).load;
  }

  function Preload() {
    var preload = usePreload();
    useIdleCallback(preload);
    return null;
  }

  function Prefetch() {
    var preload = usePreload();
    useEffect(function () {
      preload();
    }, [preload]);
    return null;
  }

  var FinalComponent = Root;
  Reflect.defineProperty(FinalComponent, 'resolver', {
    value: resolver,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Provider', {
    value: Provider,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Consumer', {
    value: Consumer,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Context', {
    value: Context,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Preload', {
    value: Preload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Prefetch', {
    value: Prefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'KeepFresh', {
    value: Preload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePreload', {
    value: usePreload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePrefetch', {
    value: usePreload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'useKeepFresh', {
    value: usePreload,
    writable: false
  });
  return FinalComponent;
}