"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createAsyncComponent = createAsyncComponent;

var _react = _interopRequireWildcard(require("react"));

var _async = require("@shopify/async");

var _reactIntersectionObserver = require("@shopify/react-intersection-observer");

var _reactIdle = require("@shopify/react-idle");

var _reactHydrate = require("@shopify/react-hydrate");

var _hooks = require("./hooks");

var _types = require("./types");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function createAsyncComponent({
  id,
  load,
  defer,
  deferHydration,
  displayName,
  renderLoading = noopRender,
  renderError = defaultRenderError,
  usePreload: useCustomPreload = noopUse,
  usePrefetch: useCustomPrefetch = noopUse,
  useKeepFresh: useCustomKeepFresh = noopUse
}) {
  const resolver = (0, _async.createResolver)({
    id,
    load
  });
  const componentName = displayName || displayNameFromId(resolver.id);
  const deferred = defer != null;
  const progressivelyHydrated = deferHydration != null;
  const scriptTiming = deferred || progressivelyHydrated ? _types.AssetTiming.CurrentPage : _types.AssetTiming.Immediate;
  const stylesTiming = deferred ? _types.AssetTiming.CurrentPage : _types.AssetTiming.Immediate;

  function Async(props) {
    const {
      resolved: Component,
      load,
      loading,
      error
    } = (0, _hooks.useAsync)(resolver, {
      scripts: scriptTiming,
      styles: stylesTiming,
      immediate: !deferred
    });
    const {
      current: startedHydrated
    } = (0, _react.useRef)((0, _reactHydrate.useHydrationManager)().hydrated);

    if (error) {
      return renderError(error);
    }

    let loadingMarkup = null;

    if (progressivelyHydrated && !startedHydrated) {
      loadingMarkup = /*#__PURE__*/_react.default.createElement(Loader, {
        defer: deferHydration,
        load: load,
        props: props
      });
    } else if (loading) {
      loadingMarkup = /*#__PURE__*/_react.default.createElement(Loader, {
        defer: defer,
        load: load,
        props: props
      });
    }

    let contentMarkup = null;
    const rendered = Component ? /*#__PURE__*/_react.default.createElement(Component, props) : null;

    if (progressivelyHydrated && !startedHydrated) {
      contentMarkup = rendered ? /*#__PURE__*/_react.default.createElement(_reactHydrate.Hydrator, {
        id: resolver.id
      }, rendered) : /*#__PURE__*/_react.default.createElement(_reactHydrate.Hydrator, {
        id: resolver.id
      });
    } else if (loading) {
      contentMarkup = renderLoading(props);
    } else {
      contentMarkup = rendered;
    }

    return /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, contentMarkup, loadingMarkup);
  }

  Async.displayName = `Async(${componentName})`;

  function usePreload(props) {
    const {
      load
    } = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    });
    const customPreload = useCustomPreload(props);
    return (0, _react.useCallback)(() => {
      load();

      if (customPreload) {
        customPreload();
      }
    }, [load, customPreload]);
  }

  function usePrefetch(props) {
    const {
      load
    } = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    });
    const customPrefetch = useCustomPrefetch(props);
    return (0, _react.useCallback)(() => {
      load();

      if (customPrefetch) {
        customPrefetch();
      }
    }, [load, customPrefetch]);
  }

  function useKeepFresh(props) {
    const {
      load
    } = (0, _hooks.useAsync)(resolver, {
      assets: _types.AssetTiming.NextPage
    });
    const customKeepFresh = useCustomKeepFresh(props);
    return (0, _react.useCallback)(() => {
      load();

      if (customKeepFresh) {
        customKeepFresh();
      }
    }, [load, customKeepFresh]);
  }

  function Preload(options) {
    (0, _reactIdle.useIdleCallback)(usePreload(options));
    return null;
  }

  Preload.displayName = `Async.Preload(${displayName})`;

  function Prefetch(options) {
    const prefetch = usePrefetch(options);
    (0, _react.useEffect)(() => {
      prefetch();
    }, [prefetch]);
    return null;
  }

  Prefetch.displayName = `Async.Prefetch(${displayName})`;

  function KeepFresh(options) {
    (0, _reactIdle.useIdleCallback)(useKeepFresh(options));
    return null;
  }

  KeepFresh.displayName = `Async.KeepFresh(${displayName})`;
  const FinalComponent = Async;
  Reflect.defineProperty(FinalComponent, 'resolver', {
    value: resolver,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Preload', {
    value: Preload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Prefetch', {
    value: Prefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'KeepFresh', {
    value: KeepFresh,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePreload', {
    value: usePreload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePrefetch', {
    value: usePrefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'useKeepFresh', {
    value: useKeepFresh,
    writable: false
  });
  return FinalComponent;
}

function noopUse() {
  return () => {};
}

function noopRender() {
  return null;
}

const DEFAULT_DISPLAY_NAME = 'Component';
const FILENAME_REGEX = /([^/]*)\.\w+$/;

function displayNameFromId(id) {
  if (!id) {
    return DEFAULT_DISPLAY_NAME;
  }

  const match = FILENAME_REGEX.exec(id);
  return match ? match[1] : DEFAULT_DISPLAY_NAME;
}

function defaultRenderError(error) {
  if (error) {
    throw error;
  }

  return null;
}

function Loader({
  defer,
  load,
  props
}) {
  const handleIntersection = (0, _react.useCallback)(({
    isIntersecting = true
  }) => {
    if (isIntersecting) {
      load();
    }
  }, [load]);
  (0, _react.useEffect)(() => {
    if (defer == null || defer === _async.DeferTiming.Mount) {
      load();
    } else if (typeof defer === 'function' && defer(props)) {
      load();
    }
  }, [defer, load, props]);

  if (typeof defer === 'function') {
    return null;
  }

  switch (defer) {
    case _async.DeferTiming.Idle:
      return /*#__PURE__*/_react.default.createElement(_reactIdle.OnIdle, {
        perform: load
      });

    case _async.DeferTiming.InViewport:
      return /*#__PURE__*/_react.default.createElement(_reactIntersectionObserver.IntersectionObserver, {
        threshold: 0,
        onIntersectionChange: handleIntersection
      });

    default:
      return null;
  }
}